import json, requests
from tools_lyc7456.uCloudapi.signature import verify_ac

# Write by lyc at 2020-12-4
# Update by lyc at 2021-10-5：优化成包


class UNI():
    """[优刻得uni虚拟网卡 api封装]
    
    """
    def __init__(self, key):
        self.PublicKey = key['PublicKey']           # 公钥
        self.PrivateKey = key['PrivateKey']         # 私钥
        self.api_url = key['api_url']               # api接口url
        self.project_id = key['project_id']         # 项目id
        self.region = key['UHOST']['region']        # 地域
        self.zone = key['UHOST']['zone']            # 可用区
        self.vpc_id = key['UHOST']['vpc_id']        # vpc id
        self.subnet_id = key['UHOST']['subnet_id']  # subnet id


    def create_networkinterface(self):
        """[创建虚拟网卡]

        Returns:
            [dict]: [response]
        """
        params = {
            'Action': 'CreateNetworkInterface',  # 接口名称
            'Region': self.region,
            'VPCId': self.vpc_id,
            'SubnetId': self.subnet_id,
            'ProjectId': self.project_id,
            'PublicKey': self.PublicKey,
        }
        # 签名
        signature = verify_ac(self.PublicKey, self.PrivateKey, params)
        params['Signature'] = signature

        # POST请求接口
        headers = {'Content-Type': 'application/json'}
        response = requests.post(url=self.api_url, headers=headers, data=json.dumps(params))
        response_dic = json.loads(response.content.decode('utf-8'))
        
        return response_dic



    def delete_networkinterface(self, uni_id):
        """[删除虚拟网卡]

        Args:
            uni_id ([string]): [虚拟网卡id]

        Returns:
            [dict]: [response]
        """
        params = {
            'Action': 'DeleteNetworkInterface',  # 接口名称
            'Region': self.region,
            'InterfaceId': uni_id,
            'ProjectId': self.project_id,
            'PublicKey': self.PublicKey,
        }
        # 签名
        signature = verify_ac(self.PublicKey, self.PrivateKey, params)
        params['Signature'] = signature

        # POST请求接口
        headers = {'Content-Type': 'application/json'}
        response = requests.post(url=self.api_url, headers=headers, data=json.dumps(params))
        response_dic = json.loads(response.content.decode('utf-8'))
        
        return response_dic



    def grantfirewall(self, uni_id, fw_id, resource_type='uhost'):
        """[应用防火墙 - GrantFirewall]https://docs.ucloud.cn/api/unet-api/grant_firewall

        Args:
            uni_id ([string]): [虚拟网卡资源ID]
            fw_id ([string]): [防火墙资源ID]
            resource_type (str, optional): [弹性IP请求绑定的资源类型]. 枚举值为: uhost: 云主机; ulb, 负载均衡器; uni：虚拟网卡; 默认值 'uhost'.

        Returns:
            [dict]: [response]
        """
        params = {
            'Action': 'GrantFirewall',  # 接口名称
            'Region': self.region,
            'FWId': fw_id,
            'ResourceType': resource_type,
            'ResourceId': uni_id,
            'ProjectId': self.project_id,
            'PublicKey': self.PublicKey,
        }
        # 签名
        signature = verify_ac(self.PublicKey, self.PrivateKey, params)
        params['Signature'] = signature

        # POST请求接口
        headers = {'Content-Type': 'application/json'}
        response = requests.post(url=self.api_url, headers=headers, data=json.dumps(params))
        response_dic = json.loads(response.content.decode('utf-8'))
        
        return response_dic



    def attach_networkinterface(self, uni_id, uhost_id):
        """[虚拟网卡绑定云主机]

        Args:
            uni_id ([string]): [虚拟网卡id]
            uhost_id ([string]): [云主机资源id]

        Returns:
            [dict]: [response]
        """
        params = {
            'Action': 'AttachNetworkInterface',  # 接口名称
            'Region': self.region,
            'InterfaceId': uni_id,
            'InstanceId': uhost_id,
            'ProjectId': self.project_id,
            'PublicKey': self.PublicKey,
        }
        # 签名
        signature = verify_ac(self.PublicKey, self.PrivateKey, params)
        params['Signature'] = signature

        # POST请求接口
        headers = {'Content-Type': 'application/json'}
        response = requests.post(url=self.api_url, headers=headers, data=json.dumps(params))
        response_dic = json.loads(response.content.decode('utf-8'))
        
        return response_dic



    def describe_networkinterface(self, uhost_id):
        '''获取网卡绑定信息'''
        params = {
            'Action': 'DescribeNetworkInterface',  # 接口名称
            'Region': self.region,
            'Limit': 1000,  # 1000个网卡内可以获取到，1000个以上需要重写递归循环
            'Offset': 0,
            'ProjectId': self.project_id,
            'PublicKey': self.PublicKey,
        }
        # 签名
        signature = verify_ac(self.PublicKey, self.PrivateKey, params)
        params['Signature'] = signature

        # POST请求接口
        headers = {'Content-Type': 'application/json'}
        response = requests.post(url=self.api_url, headers=headers, data=json.dumps(params))
        response_dic = json.loads(response.content.decode('utf-8'))

        return response_dic



    def detach_networkinterface(self, uni_id, uhost_id):
        """[虚拟网卡解绑云主机]

        Args:
            uni_id ([string]): [虚拟网卡id]
            uhost_id ([string]): [云主机资源id]

        Returns:
            [dict]: [response]
        """
        params = {
            'Action': 'DetachNetworkInterface',  # 接口名称
            'Region': self.region,
            'InterfaceId': uni_id,
            'InstanceId': uhost_id,
            'ProjectId': self.project_id,
            'PublicKey': self.PublicKey,
        }
        # 签名
        signature = verify_ac(self.PublicKey, self.PrivateKey, params)
        params['Signature'] = signature

        # POST请求接口
        headers = {'Content-Type': 'application/json'}
        response = requests.post(url=self.api_url, headers=headers, data=json.dumps(params))
        response_dic = json.loads(response.content.decode('utf-8'))
        
        return response_dic



    



import base64
import json, requests
from tools_lyc7456.uCloudapi.signature import verify_ac

# Write by lyc at 2020-12-4
# Update by lyc at 2021-2-4：增加绑定云主机告警模板id
# Update by lyc at 2021-10-4：优化成包


class UHOST():
    '''优刻得uhost云主机 api封装'''
    def __init__(self, key):
        self.PublicKey = key['PublicKey']                           # 公钥
        self.PrivateKey = key['PrivateKey']                         # 私钥
        self.api_url = key['api_url']                               # api接口url
        self.project_id = key['project_id']                         # 项目id
        self.region = key['UHOST']['region']                        # 地域
        self.zone = key['UHOST']['zone']                            # 可用区
        self.image_id = key['UHOST']['image_id']                    # 镜像id
        self.uhost_password = key['UHOST']['uhost_password']        # 密码
        self.uhost_name = key['UHOST']['uhost_name']                # 云主机名称
        self.eip_bandwidth = key['UHOST']['eip_bandwidth']          # 带宽
        self.eip_paymode = key['UHOST']['eip_paymode']              # 带宽付费类型
        self.fw_id = key['UHOST']['fw_id']                          # 防火墙id
        self.alarm_template_id = key['UHOST']['alarm_template_id']  # 云主机告警模板id



    def describe_uhostinstance(self, uhost_id):
        """[获取主机信息 - DescribeUHostInstance]https://docs.ucloud.cn/api/uhost-api/describe_uhost_instance

        Args:
            uhost_id ([string]): [UHost实例ID]

        Returns:
            [dict]: [response]
        """
        params = {
            'Action': 'DescribeUHostInstance',      # 接口名称
            'Region': self.region,
            'Zone': self.zone,
            'UHostIds.0': uhost_id,
            'ProjectId': self.project_id,
            'PublicKey': self.PublicKey,
        }
        # 签名
        signature = verify_ac(self.PublicKey, self.PrivateKey, params)
        params['Signature'] = signature

        # POST请求接口
        headers = {'Content-Type': 'application/json'}
        response = requests.post(url=self.api_url, headers=headers, data=json.dumps(params))
        response_dic = json.loads(response.content.decode('utf-8'))

        return response_dic



    def create_uhostinstance(self, uhost_cpu=1, uhost_memory=2048, uhost_disk0_size=20, uhost_disks_0_type='LOCAL_NORMAL',
                                   uhost_tag='Default', uhost_chargetype='Month', uhost_count=1, eip_operatorname='Bgp', features_uni=False):
        """[创建云主机 - CreateUHostInstance]https://docs.ucloud.cn/api/uhost-api/create_uhost_instance

        Args:
            uhost_cpu (int, optional): [虚拟CPU核数]. Defaults to 1.
            uhost_memory (int, optional): [虚拟内存大小]. Defaults to 2048.
            uhost_disk0_size (int, optional): [系统盘大小]. Defaults to 20.
            uhost_disks_0_type (str, optional): [系统盘类型]. Defaults to 'LOCAL_NORMAL'. 磁盘类型参考：https://docs.ucloud.cn/api/uhost-api/disk_type
            uhost_tag (str, optional): [业务组]. Defaults to 'Default'.
            uhost_chargetype (str, optional): [计费模式]. Defaults to 'Month'.
            uhost_count (int, optional): [本次最大创建主机数量]. Defaults to 1.
            eip_operatorname (str, optional): [弹性IP线路]. "International" 国际线路；"Bgp" 国内IP。默认为 "Bgp".
            features_uni (bool): [弹性IP线路]. 弹性网卡特性。开启了弹性网卡权限位，此特性才生效。默认 False 不开启。

        Returns:
            [type]: [description]
        """
        self.uhost_password = (base64.b64encode(self.uhost_password.encode("utf-8"))).decode('utf-8')   # 云主机密码使用base64进行编码
        params = {
            'Action': 'CreateUHostInstance',      # 接口名称
            'Region': self.region,
            'Zone': self.zone,
            'ImageId': self.image_id,
            'Password': self.uhost_password,
            'Disks.0.Type': uhost_disks_0_type,
            'Disks.0.IsBoot': 'True',
            'Disks.0.Size': uhost_disk0_size,
            'LoginMode': 'Password',
            'Name': self.uhost_name,
            'Tag': uhost_tag,
            'ChargeType': uhost_chargetype,
            'CPU': uhost_cpu,
            'Memory': uhost_memory,
            'SecurityGroupId': self.fw_id,
            'MaxCount': uhost_count,
            'NetworkInterface.0.EIP.Bandwidth': self.eip_bandwidth,
            'NetworkInterface.0.EIP.PayMode': self.eip_paymode,
            'NetworkInterface.0.EIP.OperatorName': eip_operatorname,
            'AlarmTemplateId': self.alarm_template_id,
            'ProjectId': self.project_id,
            'PublicKey': self.PublicKey,
            'Features.UNI':  features_uni,
        }
        # 签名
        signature = verify_ac(self.PublicKey, self.PrivateKey, params)
        params['Signature'] = signature

        # POST请求接口
        headers = {'Content-Type': 'application/json'}
        response = requests.post(url=self.api_url, headers=headers, data=json.dumps(params))
        response_dic = json.loads(response.content.decode('utf-8'))

        return response_dic



    def stop_uhostinstance(self, uhost_id):
        """[关闭主机 - StopUHostInstance]https://docs.ucloud.cn/api/uhost-api/stop_uhost_instance

        Args:
            uhost_id ([string]): [UHost实例ID]

        Returns:
            [dict]: [response]
        """
        params = {
            'Action': 'StopUHostInstance',      # 接口名称
            'Region': self.region,
            'Zone': self.zone,
            'UHostId': uhost_id,
            'ProjectId': self.project_id,
            'PublicKey': self.PublicKey,
        }
        # 签名
        signature = verify_ac(self.PublicKey, self.PrivateKey, params)
        params['Signature'] = signature

        # POST请求接口
        headers = {'Content-Type': 'application/json'}
        response = requests.post(url=self.api_url, headers=headers, data=json.dumps(params))
        response_dic = json.loads(response.content.decode('utf-8'))
        
        return response_dic



    def terminate_uhostinstance(self, uhost_id):
        """[删除云主机 - TerminateUHostInstance]https://docs.ucloud.cn/api/uhost-api/terminate_uhost_instance

        Args:
            uhost_id ([string]): [UHost实例ID]

        Returns:
            [dict]: [response]
        """
        params = {
            'Action': 'TerminateUHostInstance',      # 接口名称
            'Region': self.region,
            'UHostId': uhost_id,
            'ReleaseEIP': True,
            'ReleaseUDisk': True,
            'ProjectId': self.project_id,
            'PublicKey': self.PublicKey,
        }
        # 签名
        signature = verify_ac(self.PublicKey, self.PrivateKey, params)
        params['Signature'] = signature

        # POST请求接口
        headers = {'Content-Type': 'application/json'}
        response = requests.post(url=self.api_url, headers=headers, data=json.dumps(params))
        response_dic = json.loads(response.content.decode('utf-8'))
        
        return response_dic





import json
from tencentcloud.common import credential
from tencentcloud.common.profile.client_profile import ClientProfile
from tencentcloud.common.profile.http_profile import HttpProfile
from tencentcloud.common.exception.tencent_cloud_sdk_exception import TencentCloudSDKException
from tencentcloud.cvm.v20170312 import cvm_client, models

# Write by lyc at 2021-10-9

class CVM():
    """[腾讯云CVM云主机 Python3 SDK方法封装]
    """
    def __init__(self, key) -> None:
        self.SecretId = key['SecretId']             # 公钥
        self.SecretKey = key['SecretKey']           # 私钥
        self.endpoint = key['CVM']['endpoint']      # 接口地址
        self.Region = key['Region']                 # 地域
        self.Zone = key['Zone']                     # 可用区
        # CVM
        self.Password = key['CVM']['Password']                      # 实例密码
        self.InstanceName = key['CVM']['InstanceName']              # 实例名称
        self.InstanceType = key['CVM']['InstanceType']              # 机型
        self.ImageId = key['CVM']['ImageId']                        # 镜像id
        self.InstanceChargeType = key['CVM']['InstanceChargeType']  # 实例计费类型
        # VPC
        self.VpcId = key['VPC']['VpcId']                                        # VPC id
        self.SubnetId = key['VPC']['SubnetId']                                  # 子网 id
        self.InternetMaxBandwidthOut = key['VPC']['InternetMaxBandwidthOut']    # EIP带宽值
        self.SecurityGroupIds = key['VPC']['SecurityGroupIds']                  # 安全组id（列表）
        self.InternetChargeType = key['VPC']['InternetChargeType']              # EIP计费类型
        # 签名
        self.cred = credential.Credential(self.SecretId, self.SecretKey)
        httpProfile = HttpProfile()
        httpProfile.endpoint = self.endpoint

        clientProfile = ClientProfile()
        clientProfile.httpProfile = httpProfile
        self.client = cvm_client.CvmClient(self.cred, self.Region, clientProfile)


    def describeAccountQuota(self):
        """[查询用户配额详情]https://cloud.tencent.com/document/api/213/55628

        Returns:
            [json]: [json response]
        """
        try:
            req = models.DescribeAccountQuotaRequest()
            params = {
                "Filters": [
                    {
                        "Name": "zone",
                        "Values": [ self.Zone, ],
                    }
                ]
            }
            req.from_json_string(json.dumps(params))

            resp = self.client.DescribeAccountQuota(req)
            # print(resp.to_json_string())
            return resp.to_json_string()

        except TencentCloudSDKException as err:
            # print(err)
            return err



    def describeZones(self):
        """[查询可用区列表]https://cloud.tencent.com/document/api/213/15707

        Returns:
            [json]: [json response]
        """
        try:
            req = models.DescribeZonesRequest()
            params = {

            }
            req.from_json_string(json.dumps(params))

            resp = self.client.DescribeZones(req)
            # print(resp.to_json_string())
            return resp.to_json_string()

        except TencentCloudSDKException as err:
            # print(err)
            return err



    def describeInstanceTypeConfigs(self, zone, instance_family):
        """[查询实例机型列表]https://cloud.tencent.com/document/api/213/15749

        Args:
            zone ([str]): [按照【可用区】进行过滤。可用区形如：ap-guangzhou-1。]

        Returns:
            [json]: [json response]
        """
        try:
            req = models.DescribeInstanceTypeConfigsRequest()
            params = {
                "Filters": [
                    {
                        "Name": "zone",
                        "Values": [ zone, ]
                    },
                    {
                        "Name": "instance-family",
                        "Values": [ instance_family, ],
                    }
                ]
            }
            req.from_json_string(json.dumps(params))

            resp = self.client.DescribeInstanceTypeConfigs(req)
            # print(resp.to_json_string())
            return resp.to_json_string()

        except TencentCloudSDKException as err:
            # print(err)
            return err



    def runInstances(self, InstanceCount=1):
        """[创建实例]https://cloud.tencent.com/document/api/213/15730

        Args:
            InstanceCount (int, optional): [购买实例数量。包年包月实例取值范围：[1，300]，按量计费实例取值范围：[1，100]。]. 默认取值：1                      
        
        Returns:
            [json]: [json response]
        """
        try:
            req = models.RunInstancesRequest()
            params = {
                "InstanceChargeType": self.InstanceChargeType,
                "Placement": {
                    "Zone": self.Zone
                },
                "InstanceType": self.InstanceType,
                "ImageId": self.ImageId,
                "VirtualPrivateCloud": {
                    "VpcId": self.VpcId,
                    "SubnetId": self.SubnetId
                },
                "InternetAccessible": {
                    "InternetChargeType": self.InternetChargeType,
                    "InternetMaxBandwidthOut": self.InternetMaxBandwidthOut,
                    "PublicIpAssigned": True
                },
                "InstanceCount": InstanceCount,
                "InstanceName": self.InstanceName,
                "LoginSettings": {
                    "Password": self.Password
                },
                "SecurityGroupIds": self.SecurityGroupIds
            }
            req.from_json_string(json.dumps(params))

            resp = self.client.RunInstances(req)
            # print(resp.to_json_string())
            return resp.to_json_string()

        except TencentCloudSDKException as err:
            # print(err)
            return err



    def stopInstances(self, InstanceIds=[], ForceStop=True):
        """[关闭实例]https://cloud.tencent.com/document/api/213/15743

        Args:
            InstanceIds (list): [实例ID]. 
            ForceStop (bool, optional): [是否在正常关闭失败后选择强制关闭实例
                                            TRUE：表示在正常关闭失败后进行强制关闭
                                            FALSE：表示在正常关闭失败后不进行强制关闭]. Defaults to True.
        
        Returns:
            [json]: [json response]
        """
        try:
            req = models.StopInstancesRequest()
            params = {
                "InstanceIds": InstanceIds,
                "ForceStop": ForceStop
            }
            req.from_json_string(json.dumps(params))

            resp = self.client.StopInstances(req)
            # print(resp.to_json_string())
            return resp.to_json_string()

        except TencentCloudSDKException as err:
            # print(err)
            return err



    def terminateInstances(self, InstanceIds=[]):
        """[退还实例]https://cloud.tencent.com/document/api/213/15723

        Args:
            InstanceIds (list, optional): [实例ID]. 

        Returns:
            [json]: [json response]
        """
        try:
            req = models.TerminateInstancesRequest()
            params = {
                "InstanceIds": InstanceIds
            }
            req.from_json_string(json.dumps(params))

            resp = self.client.TerminateInstances(req)
            # print(resp.to_json_string())
            return resp.to_json_string()

        except TencentCloudSDKException as err:
            # print(err)
            return err



    def describeInstances(self, InstanceIds=[]):
        """[查看实例列表]https://cloud.tencent.com/document/api/213/15728

        Args:
            InstanceIds (list, optional): [实例ID列表].

        Returns:
            [json]: [json response]
        """
        try:
            req = models.DescribeInstancesRequest()
            params = {
                "InstanceIds": InstanceIds,
            }
            req.from_json_string(json.dumps(params))

            resp = self.client.DescribeInstances(req)
            # print(resp.to_json_string())
            return resp.to_json_string()

        except TencentCloudSDKException as err:
            # print(err)
            return err


